/*===========================================================================*\

   $Id$

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:
 
   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#ifndef POINTCOLLISIONALGO_H
#define POINTCOLLISIONALGO_H

#include "CollisionAlgo.h"
#include "GameEngine.h"
#include <spug/Tracer.h>

/**
 * The simplest collision algorithm, checks for the collision of the point
 * defined by its position.
 */
class PointCollisionAlgo : public CollisionAlgo {

   private:
      static spug::Tracer::Level collision;

   protected:
      /**
       * Returns true if the ray defined by "position" and "direction"
       * intersects the polygon "poly"
       * @param distance if the ray intersects, this is set to the distance
       *                 from the position to the point of intersection
       */
      bool PointCollisionAlgo::checkPolygonCollision(const Vector &position,
                                                     const Vector &direction,
                                                     const Poly &poly,
                                                     Float &distance
                                                     ) const;

      /**
       * Returns true if the ray defined by "position" and "diection"
       * intersects the axis box "box"
       * @param distance if the ray intersects, this is set to the distance
       *                 from the position to the point of intersection
       */
      bool PointCollisionAlgo::checkAxisBoxCollision(const Vector &position,
                                                     const Vector &direction,
                                                     const AxisBox &box,
                                                     Float &distance
                                                     ) const;

      /**
       * Returns true if a ray originating at coordinate "pos" and traveling
       * in direction "dir" intersects with either "min" or "max".  returns
       * the distance to the intersection in "dist".
       */
      bool PointCollisionAlgo::intersectsAxisPlane(Float pos, Float dir, 
                                                   Float min, 
                                                   Float max, 
                                                   Float &dist
                                                   ) const;

      /**
       * Returns true if min <= (pos + dir * dist) <= max.  In other words, if
       * a ray originating at the coordinate "pos" and traveling in direction
       * "dir" for distance "dist" terminates at a point within "min" and
       * "max"
       */
      static bool PointCollisionAlgo::inRange(Float pos, Float dir, 
                                              Float dist, 
                                              Float min,
                                              Float max
                                              );
   public:
      virtual bool checkForCollision(const Vector &position,
                                     const Vector &velocity,
                                     const Collidable &other,
                                     Float &distance
                                     ) const;

};

#endif
