/*===========================================================================*\

   $Id$

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:
 
   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#ifndef SIMPLECOLLIDABLE_H
#define SIMPLECOLLIDABLE_H

#include <spug/RCPtr.h>
#include "AxisBox.h"
#include "Collidable.h"

class Moveable;

SPUG_RCPTR(PolyObj);
SPUG_RCPTR(CollisionAlgo);
SPUG_RCPTR(SimpleCollidable);

/**
 * Implements collidable by allowing you to explicitly set all of the
 * collision models.
 */
class SimpleCollidable : public Collidable {
   private:
      Types types;
      PolyObjPtr polyObj;
      AxisBox box;
      Sphere sphereObj;
      Moveable *moveable;
      int priority;

   public:
      SimpleCollidable(const AxisBox &box);

      /**
       * Defines or resets the polygon collision object from the given polygon
       * object.
       */
      void setPoly(const PolyObjPtr &polyObj);

      /** Resets the axis box collision object. */
      void setAxisBox(const AxisBox &newBox);

      /**
       * Defines or resets the sphere object from the given sphere.
       */
      void setSphere(const Collidable::Sphere &sphere);

      /**
       * Sets the associated moveable.  Note that moveable is not managed -
       * moveables own SimpleCollidables, and we don't want a ref cycle.  This
       * means that Moveable must set the moveable pointer of all of its
       * collidables to null on destruction.
       */
      void setMoveable(Moveable *newMoveable) {
         moveable = newMoveable;
      }

      void setPriority(int newPriority) {
         priority = newPriority;
      }

      // implements Collidable interface
      virtual Types getTypes() const { return types; }
      virtual int getPriority() const;
      virtual bool checkForCollision(const CollidablePtr &other, 
				     Float &time) const;
      virtual void informOfCollision(const CollidablePtr &other,
				     Float distance
				     );
      virtual const Poly &getPoly() const;
      virtual AxisBox getAxisBox() const;
      virtual const Sphere &getSphere() const;
      virtual Vector getVelocity() const;
};

#endif
