#==============================================================================
#
#  $Id$
#
"""
   The wurld bootstrapping script.
"""
#
#  Copyright (C) 2007 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#==============================================================================

import wurld
import sys, os, time
sys.path.insert(0, '.')
import turtle
from util import TextPane

# set the display resolution
wurld.getDisplayDriver().setDisplaySize(640, 480)

class Event:
   """
      A scheduled event.
   """

   def __init__(self, time, target = None):
      self.time = time
      self.target = target

   def run(self):
      self.target()

# global callback - this gets called ten times per second
firstFrame = True
screenText = None
schedule = []
def globalCallback(engine, obj):
   global firstFrame, screenText, schedule
   if firstFrame:
      # turn on lighting (we have to do this _after_ all of the graphics is
      # initialized)
      wurld.getGraphics().enableLighting(True)
      firstFrame = False

      # create a new text pane occupying the top tenth of the screen
      screenText = TextPane(-1, 1, 2, 0.1)
      screenText.write("Welcome to Wurld version %s...\n" % wurld.getVersion())

      # display FPS
      engine = wurld.getGameEngine()
      engine.addHUDSticker(wurld.createFPSDisplay(engine))

      # test of exceptions
      try:
         engine.removeHUDSticker(wurld.createFPSDisplay(engine))
         assert 0, 'exceptions are broken'
      except RuntimeError, ex:
         assert ex[0] == 'NotFoundError: HUD Object not found for removal'

   # run the next event on the queue
   while schedule and schedule[0].time < time.time():
      schedule.pop(0).run()

obj = wurld.createPythonObject()
obj.setEngineFrameCallback(globalCallback)
wurld.getGameEngine().addGameObject(obj)

# add everything in the gameobjs directory
for file in os.listdir('gameobjs'):
   # ... except any pesky maintenence files
   if not file.startswith('.'):
      execfile(os.path.join('gameobjs', file), {})

helpText = None
def showHelp():
   global helpText
   helpText = TextPane(-0.5, 0.5, 1, 1)
   helpText.write('Use "z", "x", "c", and "d" to move\n'
                  '"a" fires a missile\n'
                  '"l" toggles lighting\n'
                  '"q" to quit\n'
                  '"g" to grab the mouse - Use the mouse to look around\n'
                  '"h" to show (or hide) this help message\n'
                  )

def hideHelp():
   global helpText
   helpText = None

def toggleHelp(key):
   global helpText
   print key
   if helpText:
      hideHelp()
   else:
      showHelp()

t = time.time()
schedule.append(Event(t + 1, showHelp))
schedule.append(Event(t + 6, hideHelp))

# bind the help key
wurld.getDisplayDriver().getKeyHandler().bind(ord('h'), 
                                              wurld.createKeyCommand(toggleHelp,
                                                                     None
                                                                     )
                                              )
print '=' * 78
print 'Finished wurldrc.py'
print '=' * 78
